@@ -160,6 +160,10 @@ public:
 	virtual TeAttrTableVector& attrTables () 
 	{ return attTables_; }
 
+  //! Returns a vector with the definition of all attribute tables associated to layer
+	virtual const TeAttrTableVector& attrTables () const
+	{ return attTables_; }
+
 	//! Creates a new attribute table for the layer
 	virtual bool createAttributeTable(TeTable& table);
 
@@ -418,6 +422,9 @@ public:
 	virtual TeRepresPointerVector& vectRepres()
 	{	return repVector_; }
 
+  virtual const TeRepresPointerVector& vectRepres() const
+	{	return repVector_; }
+
 	//!< Returns the dateTime that the layer was last edited.
 	virtual TeTime getEditionTime() const;
 
