@@ -56,16 +56,16 @@ TeDatum  TeGetDatumFromProj4(const string& proj4);
 const char* teProjInfo[]= {
 "Albers",					"1",  "1",  "1", "1",  "1",  "0", "1", "1", 
 "LatLong",					"1",  "0",  "0", "0",  "0",  "0", "0", "0",   
-"LambertConformal",			        "1",  "1",  "1", "1",  "1",  "0", "1", "1", 
+"LambertConformal",			"1",  "1",  "1", "1",  "1",  "0", "1", "1", 
 "Mercator",					"1",  "1",  "1", "1",  "0",  "0", "1", "1", 
 "Miller",					"1",  "1",  "0", "0",  "0",  "0", "1", "1", 
 "UTM",						"1",  "1",  "0", "0",  "0",  "1", "1", "1", 
-"Polyconic",				        "1",  "1",  "1", "0",  "0",  "0", "1", "1",
-"Sinusoidal",				        "1",  "1",  "0", "0",  "0",  "0", "1", "1",
-"CylindricalEquidistant",	                "1",  "1",  "0", "1",  "0",  "0", "1", "1",
-"PolarStereographic",		                "1",  "1",  "0", "0",  "0",  "0", "1", "1",
-"Satellite",		                        "1",  "1",  "1", "0",  "0",  "0", "1", "1",
-"NoProjection",				        "1",  "0",  "0", "0",  "0",  "0", "0", "0"		
+"Polyconic",				"1",  "1",  "1", "0",  "0",  "0", "1", "1",
+"Sinusoidal",				"1",  "1",  "0", "0",  "0",  "0", "1", "1",
+"CylindricalEquidistant",	"1",  "1",  "0", "1",  "0",  "0", "1", "1",
+"PolarStereographic",		"1",  "1",  "0", "0",  "0",  "0", "1", "1",
+"Satellite",		        "1",  "1",  "1", "0",  "0",  "0", "1", "1",
+"NoProjection",				"1",  "0",  "0", "0",  "0",  "0", "0", "0"		
 };
 
 
@@ -170,6 +170,166 @@ TeProjectionFactory::make ( const TeProjectionParams& par )
 		return new TeNoProjection(par.datum,punits);	
 	return 0;
 }
+  
+TeProjection*
+TeProjectionFactory::make (int epsgcode)
+{
+	// geographic
+	int datumcode;
+	if (epsgcode >= 4208 && epsgcode <=4979)
+	{ 
+		if (epsgcode == 4618)
+			datumcode = 6618;
+		else if (epsgcode == 4979)
+		   datumcode = 6326;
+		 else if (epsgcode == 4225)
+		   datumcode = 6225;
+		 else if (epsgcode == 4267)
+		   datumcode = 6609;
+		 else if (epsgcode == 4269)
+		   datumcode = 6152;
+		else if (epsgcode == 4208)	   
+		   datumcode = 6208;
+		/*else if (epsgcode == indian)			   
+		   datumcode = 6240;*/
+		else if (epsgcode == 4240)
+		   datumcode = 6240;
+		else if (epsgcode == 4224)
+		   datumcode = 6224;		   
+		/*else if (datum.name() == "Indian")
+		{
+			epsgcode = 4240;
+		} 
+		else if (datum.name() == "Astro-Chua")
+		{
+			epsgcode = 4224;
+		}*/
+		TeDatum dat = TeDatumFactory::makeFromEPSGCode(Te2String(epsgcode+2000));
+		return new TeLatLong (dat,"DecimalDegrees");
+	}
+
+	TeProjectionParams par;	
+	
+	// ---- UTM
+	if (epsgcode >=29168 && epsgcode <= 29172)
+	{
+		par.name = "UTM";
+		par.hemisphere = TeNORTH_HEM;
+		par.datum = TeDatumFactory::make("SAD69");
+        par.lat0 = 0;
+		par.offx = 500000.0;
+		par.offy = 0;
+		par.scale = 0.999600;
+		par.lon0 = (-183. + 6.*(epsgcode-29150+1))*TeCDR; 
+	}
+	else if (epsgcode >=29187 && epsgcode <= 29195)
+	{
+		par.name = "UTM";
+		par.hemisphere = TeSOUTH_HEM;
+		par.datum = TeDatumFactory::make("SAD69");
+        par.lat0 = 0;
+		par.offx = 500000.0;
+		par.offy = 10000000.0;
+		par.scale = 0.999600;		
+		par.lon0 = (-183. + 6.*(epsgcode-29160+1))*TeCDR; 
+	}
+	else if (epsgcode >32600 && epsgcode <= 32660)
+	{
+		par.name = "UTM";
+		par.hemisphere = TeNORTH_HEM;
+		par.datum = TeDatumFactory::make("WGS84");
+        par.lat0 = 0;
+		par.offx = 500000.0;
+		par.offy = 0;
+		par.scale = 0.999600;
+		par.lon0 = (-183. + 6.*(epsgcode-32600+1))*TeCDR; 
+	}
+	else if (epsgcode >32700 && epsgcode <= 32760)
+	{
+		par.name = "UTM";
+		par.hemisphere = TeSOUTH_HEM;
+		par.datum = TeDatumFactory::make("WGS84");
+        par.lat0 = 0;
+		par.offx = 500000.0;
+		par.offy = 10000000.0;
+		par.scale = 0.999600;
+		par.lon0 = (-183. + 6.*(epsgcode-32700+1))*TeCDR; 
+	}	
+	else if (epsgcode >22520 && epsgcode <= 22525)
+	{
+		par.name = "UTM";
+		par.hemisphere = TeSOUTH_HEM;
+		par.datum = TeDatumFactory::make("CorregoAlegre");
+        par.lat0 = 0;
+		par.offx = 500000.0;
+		par.offy = 10000000.0;
+		par.scale = 0.999600;
+		par.lon0 = (-183. + 6.*(epsgcode-22520+1))*TeCDR; 
+	}
+	else if (epsgcode >26700 && epsgcode <= 26722)
+	{
+		par.name = "UTM";
+		par.hemisphere = TeNORTH_HEM;
+		par.datum = TeDatumFactory::make("NAD27");
+        par.lat0 = 0;
+		par.offx = 500000.0;
+		par.offy = 0.0;
+		par.scale = 0.999600;
+		par.lon0 = (-183. + 6.*(epsgcode-26700+1))*TeCDR; 
+	}
+	else if (epsgcode >26900 && epsgcode <= 26923)
+	{
+		par.name = "UTM";
+		par.hemisphere = TeNORTH_HEM;
+		par.datum = TeDatumFactory::make("NAD83");
+        par.lat0 = 0;
+		par.offx = 500000.0;
+		par.offy = 0.0;
+		par.scale = 0.999600;
+		par.lon0 = (-183. + 6.*(epsgcode-26900+1))*TeCDR; 
+	}
+	else if (epsgcode >=20822 && epsgcode <= 20825)
+	{
+		par.name = "UTM";
+		par.hemisphere = TeSOUTH_HEM;
+		par.datum = TeDatumFactory::make("Aratu");
+        par.lat0 = 0;
+		par.offx = 500000.0;
+		par.offy = 10000000.0;
+		par.scale = 0.999600;
+		par.lon0 = (-183. + 6.*(epsgcode-20800+1))*TeCDR; 
+	}
+	// ---- UTM
+	else if (epsgcode == 29101)
+	{
+		par.name = "Polyconic";
+		par.hemisphere = TeSOUTH_HEM;
+		par.datum = TeDatumFactory::make("SAD69");
+        par.lat0 = 0;
+		par.offx = 0.0;
+		par.offy = 0.0;
+		par.scale = 1;
+		par.lon0 = -54.0*TeCDR; 
+	}	
+	else if (epsgcode == 9823)
+	{	
+		par.name = "CylindricalEquidistant";
+		par.hemisphere = TeSOUTH_HEM;
+		par.datum = TeDatumFactory::make("Spherical");
+        par.lat0 = 0;
+		par.offx = 0.0;
+		par.offy = 0.0;
+		par.scale = 1;
+		par.lon0 = 0*TeCDR; 	
+	
+	}
+	else
+	{
+		par.name == "NoProjection";
+	}
+	return TeProjectionFactory::make(par);	
+
+}
 
 TeProjectionParams 
 TeProjection::params () const
@@ -205,6 +365,7 @@ TeProjection::TeProjection(const TeProjection& rhs)
 	GPunits  = rhs.GPunits;
 	GPscale  = rhs.GPscale;
 	GPhemisphere = rhs.GPhemisphere;
+	GPepsgCode = rhs.GPepsgCode;
 }
 
 TeProjection& 
@@ -223,6 +384,7 @@ TeProjection::operator=(const TeProjection& rhs)
 		GPunits  = rhs.GPunits;
 		GPscale  = rhs.GPscale;
 		GPhemisphere = rhs.GPhemisphere;
+		GPepsgCode = rhs.GPepsgCode;
 	}
 	return *this;
 }
@@ -521,13 +683,59 @@ TeProjection :: ChangeLL (double &lon1, double &lat1)
 	lon1 = lon2;
 }
 
+
+TeUtm::TeUtm(const TeDatum& datum, double long0, double lat0, 
+		          double offx, double offy, 
+			        const string& units,
+			        double scale, TeHemisphere hemisphere):
+	  TeProjection( "UTM", datum, long0, lat0, offx, offy, 0., 0., units, scale, hemisphere)
+
+{
+  if (hemisphere == TeNORTH_HEM)
+		GPoffy = 0.;
+		
+  int zone = ((int)((long0*TeCRD+183.0)/6.0)-1);
+  if (datum.name() == "SAD69")
+  {
+    if (hemisphere == TeNORTH_HEM)
+      GPepsgCode = 29150+zone;
+    else
+      GPepsgCode = 29170+zone;
+  }
+  else if (datum.name() == "WGS84")
+  {
+    if (hemisphere == TeNORTH_HEM)
+      GPepsgCode = 32600+zone;
+    else
+      GPepsgCode = 32700+zone;
+  }
+  else if (datum.name() == "CorregoAlegre")
+  {
+    if (hemisphere == TeSOUTH_HEM)
+      GPepsgCode = 22500+zone;
+  }
+  else if (datum.name() == "NAD27")
+  {
+     if (hemisphere == TeNORTH_HEM)
+      GPepsgCode = 26700+zone;
+  }
+  else if (datum.name() == "NAD83")
+  {
+     if (hemisphere == TeNORTH_HEM)
+      GPepsgCode = 26900+zone;
+  }
+ else if (datum.name() == "Aratu")
+ {
+     if (hemisphere == TeSOUTH_HEM)
+      GPepsgCode = 20800+zone;
+  }  
+}
+
 /********************************************************************
 		GEODETIC TO UTM COORDINATES
 ********************************************************************/
-
-
 TeCoord2D
-TeUtm :: LL2PC (TeCoord2D& ptll)
+TeUtm :: LL2PC (const TeCoord2D& ptll)
 {
 
 	double	k0,		// Scale factor
@@ -625,13 +833,12 @@ TeUtm :: LL2PC (TeCoord2D& ptll)
 	return(TeCoord2D(ptllx+GPoffx,ptlly+GPoffy));
 }
 
-
 /********************************************************************
 		UTM TO GEODETIC COORDINATES
 ********************************************************************/
 
 TeCoord2D
-TeUtm :: PC2LL (TeCoord2D& ptpc)
+TeUtm :: PC2LL (const TeCoord2D& ptpc)
 {
 	double  k0,             	// Scale factor
 		equad,			// Squared eccentricity
@@ -769,7 +976,7 @@ TeUtm :: PC2LL (TeCoord2D& ptpc)
 		GEODETIC TO MERCATOR COORDINATES
 ********************************************************************/
 TeCoord2D
-TeMercator :: LL2PC (TeCoord2D& ptll)
+TeMercator :: LL2PC (const TeCoord2D& ptll)
 {
 	double	equad,			//Squared eccentricity
 		aux1,			// Ancillary variables
@@ -822,7 +1029,7 @@ TeMercator :: LL2PC (TeCoord2D& ptll)
 		MERCATOR TO GEODETIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TeMercator :: PC2LL (TeCoord2D& ptpc)
+TeMercator :: PC2LL (const TeCoord2D& ptpc)
 {
 	double	equad,			//Squared eccentricity 
 	        pi,			// PII value
@@ -882,7 +1089,7 @@ TeMercator :: PC2LL (TeCoord2D& ptpc)
 		GEODETIC TO LAMBERT CONIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TeLambertConformal :: LL2PC (TeCoord2D& ptll)
+TeLambertConformal :: LL2PC (const TeCoord2D& ptll)
 {
 
 	double	equad,			// Squared eccentricity
@@ -931,7 +1138,7 @@ TeLambertConformal :: LL2PC (TeCoord2D& ptll)
 		LAMBERT CONIC TO GEODETIC COORDINATES
  ********************************************************************/
 TeCoord2D
-TeLambertConformal :: PC2LL (TeCoord2D& ptpc)
+TeLambertConformal :: PC2LL (const TeCoord2D& ptpc)
 {
 	double	equad,			// Squared eccentricity 
 		pi,			// PI value
@@ -991,7 +1198,7 @@ TeLambertConformal :: PC2LL (TeCoord2D& ptpc)
 		GEODETIC TO POLYCONIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TePolyconic :: LL2PC (TeCoord2D& ptll)
+TePolyconic :: LL2PC (const TeCoord2D& ptll)
 {
 	double	equad,			// Squared eccentricity 
 		n,			// Great normal of ellipsoid
@@ -1033,11 +1240,19 @@ TePolyconic :: LL2PC (TeCoord2D& ptll)
 	return(TeCoord2D(ptllx+GPoffx,ptlly+GPoffy));	
 }
 
+TePolyconic::TePolyconic (const TeDatum& datum, double long0, double lat0, double offx, double offy,
+				 const string& units): 
+	TeProjection ( "Polyconic", datum, long0, lat0, offx, offy, 0., 0., units )
+{
+	GPepsgCode = 29101;
+}
+
+	
 /*******************************************************************
 		POLYCONIC TO GEODETIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TePolyconic :: PC2LL (TeCoord2D& ptpc)
+TePolyconic :: PC2LL (const TeCoord2D& ptpc)
 {	
 	double	A,	
 		B,
@@ -1124,8 +1339,47 @@ TePolyconic :: PC2LL (TeCoord2D& ptpc)
 /********************************************************************
 	GEODETIC TO LAT LONG
 ********************************************************************/
+TeLatLong::TeLatLong( const TeDatum& datum, const string& units ):
+	   TeProjection ( "LatLong", datum, 0., 0., 0., 0., 0., 0., units )
+{
+  if (datum.name() == "SAD69")
+  {
+	GPepsgCode = 4618;
+  }
+  else if (datum.name() == "WGS84")
+  {
+	GPepsgCode = 4979;
+  }
+  else if (datum.name() == "Indian")
+  {
+	GPepsgCode = 4240;
+  } 
+  else if (datum.name() == "Astro-Chua")
+  {
+	GPepsgCode = 4224;
+  }
+  else if (datum.name() == "CorregoAlegre")
+  {
+	GPepsgCode = 4225;
+  }
+  else if (datum.name() == "NAD27")
+  {
+	GPepsgCode = 4267;
+  }
+  else if (datum.name() == "NAD83")
+  {
+	GPepsgCode = 4269;
+  }
+ else if (datum.name() == "Aratu")
+ {
+	GPepsgCode = 4208;
+  }  
+
+  
+}
+
 TeCoord2D
-TeLatLong :: LL2PC (TeCoord2D& ptll)
+TeLatLong :: LL2PC (const TeCoord2D& ptll)
 {
 	return TeCoord2D( ptll.x()*TeCRD, ptll.y()*TeCRD ); 
 }
@@ -1143,7 +1397,7 @@ TeLatLong :: LL2PC (double xi, double yi, double& xo, double& yo) const
 	LAT LONG TO GEODETIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TeLatLong :: PC2LL (TeCoord2D& ptpc)
+TeLatLong :: PC2LL (const TeCoord2D& ptpc)
 {
 	double ptpcx, ptpcy;
 
@@ -1168,12 +1422,12 @@ TeLatLong :: PC2LL (double xi, double yi, double& ptpcx, double& ptpcy)
 
 	return; 
 }
-
+	
 /********************************************************************
 	GEODETIC TO ALBERS EQUAL-AREA COORDINATES
 ********************************************************************/
 TeCoord2D
-TeAlbers :: LL2PC (TeCoord2D& ptll)
+TeAlbers :: LL2PC (const TeCoord2D& ptll)
 {
 
 	double	equad,			// Squared eccentricity
@@ -1221,7 +1475,7 @@ TeAlbers :: LL2PC (TeCoord2D& ptll)
 		ALBERS EQUAL-AREA TO GEODETIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TeAlbers :: PC2LL(TeCoord2D& ptpc)
+TeAlbers :: PC2LL(const TeCoord2D& ptpc)
 {
 
 	double	equad,			// Squared eccentricity
@@ -1291,7 +1545,7 @@ TeAlbers :: PC2LL(TeCoord2D& ptpc)
 		GEODETIC TO MILLER COORDINATES
 ********************************************************************/
 TeCoord2D
-TeMiller :: LL2PC (TeCoord2D& ptll)
+TeMiller :: LL2PC (const TeCoord2D& ptll)
 {
 
 	double	pi,		// PI value
@@ -1326,7 +1580,7 @@ TeMiller :: LL2PC (TeCoord2D& ptll)
 		MILLER TO GEODETIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TeMiller :: PC2LL (TeCoord2D& ptpc)
+TeMiller :: PC2LL (const TeCoord2D& ptpc)
 {
 
 	double	e,		// Neperian logarithm base 
@@ -1366,7 +1620,7 @@ TeMiller :: PC2LL (TeCoord2D& ptpc)
 		GEODETIC TO SINUSOIDAL COORDINATES
 ********************************************************************/
 TeCoord2D
-TeSinusoidal :: LL2PC (TeCoord2D& ptll)
+TeSinusoidal :: LL2PC (const TeCoord2D& ptll)
 {
 
 	double	ptllx,ptlly;	// Ancillary variables 
@@ -1389,7 +1643,7 @@ TeSinusoidal :: LL2PC (TeCoord2D& ptll)
 		SINUSOIDAL TO GEODETIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TeSinusoidal :: PC2LL (TeCoord2D& ptpc)
+TeSinusoidal :: PC2LL (const TeCoord2D& ptpc)
 {
 
 	double	ptpcx,ptpcy;	// Ancillary variables
@@ -1409,11 +1663,24 @@ TeSinusoidal :: PC2LL (TeCoord2D& ptpc)
 	return(TeCoord2D(ptpcx,ptpcy));
 }
 
+TeCylindricalEquidistant::TeCylindricalEquidistant (const TeDatum& datum, double lon0, double offx, 
+                                                    double offy, double stlat1, const string& units):
+			TeProjection ( "CylindricalEquidistant", datum, lon0, 0., offx, offy, stlat1, 0,units)
+{
+  if (datum.name() == "Spherical")
+  {
+	GPepsgCode = 9823;
+  }
+  else 
+  {
+	GPepsgCode = 9842;
+  }
+}
 /*******************************************************************
 		GEODETIC TO CYLINDRICAL EQUIDISTANT COORDINATES
 ********************************************************************/
 TeCoord2D
-TeCylindricalEquidistant :: LL2PC (TeCoord2D& ptll)
+TeCylindricalEquidistant :: LL2PC (const TeCoord2D& ptll)
 {
 	double	ptllx,ptlly;
 
@@ -1433,7 +1700,7 @@ TeCylindricalEquidistant :: LL2PC (TeCoord2D& ptll)
 	CYLINDRICAL EQUIDISTANT TO	GEODETIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TeCylindricalEquidistant :: PC2LL (TeCoord2D& ptpc)
+TeCylindricalEquidistant :: PC2LL (const TeCoord2D& ptpc)
 {
 	double	ptpcx,ptpcy;
 
@@ -1451,11 +1718,21 @@ TeCylindricalEquidistant :: PC2LL (TeCoord2D& ptpc)
 	return (TeCoord2D(ptpcx,ptpcy));
 }
 
+TePolarStereographic::TePolarStereographic ( const TeDatum& datum, double lon0,  double offx, 
+				 double offy, const string& units, const TeHemisphere hem ):
+	TeProjection ( "PolarStereographic", datum, lon0, 0., offx, offy, 0, 0, units, 1, hem )
+{
+	if (GPhemisphere == TeNORTH_HEM)
+		GPepsgCode = 3995;
+	else
+		GPepsgCode = 3031;
+}
+
 /*******************************************************************
 		GEODETIC TO POLAR STEREOGRAPHIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TePolarStereographic :: LL2PC (TeCoord2D& ptll)
+TePolarStereographic :: LL2PC (const TeCoord2D& ptll)
 {
 	double	k0,				// Scale factor
 		e,					// Eccentricity
@@ -1645,7 +1922,7 @@ void TePolarStereographic :: LL2PC (ostream& java) const
 	POLAR STEREOGRAPHIC TO	GEODETIC COORDINATES
 ********************************************************************/
 TeCoord2D
-TePolarStereographic :: PC2LL (TeCoord2D& ptpc)
+TePolarStereographic :: PC2LL (const TeCoord2D& ptpc)
 {
 	double	k0,			// Scale factor
 
@@ -1740,7 +2017,7 @@ TePolarStereographic :: PC2LL (TeCoord2D& ptpc)
 		GEODETIC COORDINATES TO SATELLITE PROJECTION
 *************************************************************************/
 TeCoord2D 
-TeSatelliteProjection::LL2PC(TeCoord2D& ptll)
+TeSatelliteProjection::LL2PC(const TeCoord2D& ptll)
 {
 	double equad, n, x, y, z, sn, dp, dl,xlo,yla,lin,col,resx,resy;
 	double x1, x2, a, b, c,  Rd2, Rm, Rm2, Rs2, v;
@@ -1814,7 +2091,7 @@ TeSatelliteProjection::LL2PC(TeCoord2D& ptll)
 		SATELLITE PROJECTION TO GEODETIC COORDINATES
 ************************************************************************/
 TeCoord2D
-TeSatelliteProjection::PC2LL(TeCoord2D& ptpc)
+TeSatelliteProjection::PC2LL(const TeCoord2D& ptpc)
 {
 	double	dl, dp, x, y, z, x1, x2, equad, n, d, sn,
 		a, b, c,  Rd2, Rm,Rm2, Rs2,v,ptpcx,ptpcy,
